package _default

import (
	"downloader/internal/core/entities"
	"downloader/internal/core/ports/download_request"
	"downloader/internal/core/ports/downloader"
	"downloader/internal/core/services/download"
	"fmt"
	"go.uber.org/zap"
)

type localBackgroundService struct {
	repository download_request.Repository
	logger     *zap.SugaredLogger
	downloader downloader.Downloader
}

func NewLocalBackgroundService(repository download_request.Repository, logger *zap.SugaredLogger, downloader downloader.Downloader) download.BackgroundService {
	return &localBackgroundService{repository: repository, logger: logger, downloader: downloader}
}

func (l localBackgroundService) Run(download *entities.Download) error {
	logger := l.logger.With("downloadId", download.ID)

	go func() {
		download.Status = "started"
		_ = l.repository.Update(download)

		err := l.downloader.Download(download.Link, func(progress string) {
			download.Status = fmt.Sprintf("in-progress: %s", progress)
			_ = l.repository.Update(download)
		})
		download.Status = "done"

		if err != nil {
			logger.Error(err)
			download.Status = "failed"
		}

		err = l.repository.Update(download)
		if err != nil {
			logger.Errorw("download request failed",
				"downloadLink", download.Link)
			download.Status = "failed"

			if updateErr := l.repository.Update(download); updateErr != nil {
				panic(updateErr)
			}
		} else {
			logger.Info("download request done")
		}
	}()

	return nil
}
